package fi.flexplex.lib;

public interface FlexLibAdapter {

	/**
	 * Called when WebSocket is connected and server responded HELLO
	 */
	public default void onWebSocketConnect() {}

	/**
	 * Called when WebSocket is disconnected
	 */
	public default void onWebSocketDisconnect() {}

	/**
	 * Called when player friend requests are toggle on/off
	 */
	public default void onFriendRequestsToggle(final String player, final boolean newValue) {}

	/**
	 * Called when player friend list or friend requests are updated
	 * @param player
	 */
	public default void onFriendsUpdate(final String player) {}

	/**
	 * New friend request event
	 * @param player
	 * @param targetPlayer
	 */
	public default void onFriendRequest(final String player, final String targetPlayer) {}

	/**
	 * Friend request accepted event
	 * @param player
	 * @param targetPlayer
	 */
	public default void onFriendRequestAccept(final String player, final String targetPlayer) {}

	/**
	 * Friend request deleted event
	 * @param player
	 * @param targetPlayer
	 */
	public default void onFriendRequestDelete(final String player, final String targetPlayer) {}

	/**
	 * Friend deleted event
	 * @param player
	 * @param targetPlayer
	 */
	public default void onFriendDelete(final String player, final String targetPlayer) {}

	/**
	 * Called when error happened
	 * @param message
	 */
	public default void onError(final String message) {}

}
