package fi.flexplex.lib;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.Optional;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.JsonSyntaxException;

public final class GraphQLApi {

	private static final HttpClient HTTP_CLIENT = HttpClient.newBuilder()
		.version(HttpClient.Version.HTTP_1_1)
		.build();

	private final FlexLib flexLib;
	private final URI uri;

	protected GraphQLApi(final FlexLib flexLib, final String url) {
		this.flexLib = flexLib;
		this.uri = URI.create(url);
	}

	protected Optional<JsonObject> execute(final String query) {
		final HttpResponse<String> response;
		final HttpRequest request = HttpRequest.newBuilder()
			.POST(HttpRequest.BodyPublishers.ofString("{\"query\":\"" + query.replace("\"", "\\\"") + "\"}"))
			.uri(this.uri)
			.build();

		try {
			response = HTTP_CLIENT.send(request, HttpResponse.BodyHandlers.ofString());
		} catch (final IOException e) {
			for (final FlexLibAdapter listener : this.flexLib.getEventListeners()) {
				listener.onError("Failed to send GraphQL query to FlexPlex server.");
			}
			return Optional.empty();
		} catch (final InterruptedException e) {
			Thread.currentThread().interrupt();
			return Optional.empty();
		}

		final JsonObject json;
		try {
			json = JsonParser.parseString(response.body()).getAsJsonObject();
		} catch (final JsonSyntaxException | IllegalStateException e) {
			for (final FlexLibAdapter listener : this.flexLib.getEventListeners()) {
				listener.onError("Received invalid json from FlexPlex GraphQL API");
			}
			return Optional.empty();
		}

		if (json.has("data")) {
			return Optional.of(json.get("data").getAsJsonObject());
		} else {
			return Optional.empty();
		}
	}

}
