package fi.flexplex.lib;

import java.io.IOException;
import java.net.SocketException;

import com.neovisionaries.ws.client.WebSocket;
import com.neovisionaries.ws.client.WebSocketAdapter;
import com.neovisionaries.ws.client.WebSocketException;
import com.neovisionaries.ws.client.WebSocketFactory;

public final class WebSocketClient {

	private final FlexLib flexLib;
	private final WebSocket webSocket;

	protected WebSocketClient(final FlexLib flexLib, final String url) throws IOException {
		this.flexLib = flexLib;
		this.webSocket = this.createWebSocket(url);
		try {
			this.webSocket.getConnectedSocket().setSoTimeout(0);
			this.webSocket.getConnectedSocket().setKeepAlive(true);
		} catch (final SocketException | WebSocketException e) {
			for (final FlexLibAdapter listener : this.flexLib.getEventListeners()) {
				listener.onError("Failed to set WebSocket settings");
			}
		}
	}

	protected void connect() throws WebSocketException {
		this.webSocket.connect();
	}

	protected boolean isOpen() {
		return this.webSocket.isOpen();
	}

	protected void disable() {
		this.webSocket.disconnect();
		for (final FlexLibAdapter listener : this.flexLib.getEventListeners()) {
			listener.onWebSocketDisconnect();
		}
	}

	private WebSocket createWebSocket(final String url) throws IOException {
		return new WebSocketFactory()
			.createSocket(url)
			.addHeader("Token", this.flexLib.getToken())
			.addListener(new WebSocketAdapter() {
				@Override
				public void onTextMessage(final WebSocket ws, final String message) {
					final String[] args = message.split(" ");
					switch (args[0]) {
						case "FRIEND_REQUEST_TOGGLE":
							if (args.length == 3) {
								for (final FlexLibAdapter listener : flexLib.getEventListeners()) {
									listener.onFriendRequestsToggle(args[1], args[2].equals("TRUE"));
									listener.onFriendsUpdate(args[1]);
								}
							}
							break;
						case "FRIEND_REQUEST":
							if (args.length == 3) {
								for (final FlexLibAdapter listener : flexLib.getEventListeners()) {
									listener.onFriendRequest(args[1], args[2]);
									listener.onFriendsUpdate(args[1]);
									listener.onFriendsUpdate(args[2]);
								}
							}
							break;
						case "FRIEND_ACCEPT":
							if (args.length == 3) {
								for (final FlexLibAdapter listener : flexLib.getEventListeners()) {
									listener.onFriendRequestAccept(args[1], args[2]);
									listener.onFriendsUpdate(args[1]);
									listener.onFriendsUpdate(args[2]);
								}
							}
							break;
						case "FRIEND_REQUEST_DELETE":
							if (args.length == 3) {
								for (final FlexLibAdapter listener : flexLib.getEventListeners()) {
									listener.onFriendRequestDelete(args[1], args[2]);
									listener.onFriendsUpdate(args[1]);
									listener.onFriendsUpdate(args[2]);
								}
							}
							break;
						case "FRIEND_DELETE":
							if (args.length == 3) {
								for (final FlexLibAdapter listener : flexLib.getEventListeners()) {
									listener.onFriendDelete(args[1], args[2]);
									listener.onFriendsUpdate(args[1]);
									listener.onFriendsUpdate(args[2]);
								}
							}
							break;
						case "HELLO":
							for (final FlexLibAdapter listener : flexLib.getEventListeners()) {
								listener.onWebSocketConnect();
							}
							break;
						default:
							for (final FlexLibAdapter listener : flexLib.getEventListeners()) {
								listener.onError("Received invalid WebSocket message from FlexPlex. Message: " + args[0]);
							}
							break;
					}
				}
			});
	}

}
